@@ -23,7 +23,7 @@ from ice.llutil.ignore_me import IgnoreMe
 from ice.llutil.launcher import ElasticLauncher, Events, global_shared_events
 from ice.llutil.launcher.launcher import get_current_launcher
 from ice.llutil.logger import get_logger
-from ice.llutil.utils import in_main_process, init_torch_multiprocessing
+from ice.llutil.utils import enable_auto_freeze, in_main_process, init_torch_multiprocessing
 from ice.llutil.print import _print, set_printoptions
 
 from tqdm import tqdm
@@ -75,7 +75,7 @@ class Task(_Task):
         config: a dict of configs.
     """
     @overload
-    def __init__(self, *, train: bool, steps: int, tags="*"): ...
+    def __init__(self, *, train: bool, steps: int, tags="*", simulate_epoch:bool=False): ...
 
     @overload
     def __init__(self, *, train: bool, epochs: int, tags="*"): ...
@@ -83,7 +83,7 @@ class Task(_Task):
     def __init__(self, *args, **kwds) -> None:
         super().__init__(*args, **kwds)
 
-    def __freeze__(self, *, train: bool, tags="*", **kwds):
+    def __freeze__(self, *, train: bool, tags="*", simulate_epoch=False, **kwds):
         """Freeze the task.
 
         Args:
@@ -104,6 +104,7 @@ class Task(_Task):
         self.task_epochs = 0
         self.epoch_size = 0
         self.finished = False
+        self.simulate_epoch = simulate_epoch
         return self
 
     def __call__(self, hypergraph: "HyperGraph", launcher: ElasticLauncher):
@@ -172,11 +173,16 @@ class Task(_Task):
             if launcher.local_rank == 0:
                 # update progress bar total
                 launcher.events.progress_bar_total.value = self.total_steps
+            if self.simulate_epoch:
+                self.egraph.apply("epoch_start")
             for self.epoch_steps in range(self.epoch_steps, self.total_steps):
                 try:
                     for _ in range(hypergraph.grad_acc_steps):
                         self._iterate()
                 except StopTask: return
+            if self.simulate_epoch:
+                self.egraph.apply("epoch_end")
+                self.global_auto_epochs += 1
 
 
     def _iterate(self):
@@ -442,23 +448,19 @@ class HyperGraph:
 
         self.entrypoint = None
         self.grad_acc_steps = 1
-        self.init_autocast(autocast_enabled, autocast_dtype, grad_scaler)
+        self.init_autocast(autocast_enabled, grad_scaler)
 
-    def init_autocast(self, autocast_enabled=True, autocast_dtype=None, grad_scaler:Union[bool, GradScaler] = None):
+    def init_autocast(self, autocast_enabled=True, grad_scaler:Union[bool, GradScaler] = None):
         """Initialize autocast.
 
         Args:
             autocast_enabled (bool): If True, enable autocast.
-            autocast_dtype (str): Data type to cast the gradients to.
             grad_scaler (GradScaler): Gradient scaler.
 
         Raises:
             ValueError: If the autocast_dtype is not valid.
         """
-        if autocast_dtype is None:
-            import torch
-            autocast_dtype = torch.float16
-        self.autocast_kwds = dict(enabled=autocast_enabled, dtype=autocast_dtype)
+        self.autocast_kwds = dict(enabled=autocast_enabled, dtype=None)
         self.init_grad_scaler(grad_scaler if grad_scaler is not None else autocast_enabled)
     
     def is_autocast_enabled(self) -> bool:
@@ -606,7 +608,7 @@ class HyperGraph:
         if shortcut_query not in self._shortcuts:
             for key in keys:
                 name, node, tags = self.nodes[key]
-                egraph.add_node(name, node, tags)
+                egraph.add_node(name, node.auto_freeze(), tags)
             self._shortcuts[shortcut_query] = egraph
             if self.launcher.local_rank == 0:
                 get_logger().info(f"selected nodes: {keys}")
@@ -761,7 +763,7 @@ class HyperGraph:
         
         # setup tensorboard
         from ice.llutil.board import BoardWriter
-        if get_current_launcher().rank == 0:
+        if not in_main_process() and get_current_launcher().rank == 0:
             self.board:BoardWriter = BoardWriter(self.run_info.out_dir)
         else:
             self.board:BoardWriter = IgnoreMe()
@@ -820,8 +822,6 @@ class HyperGraph:
         root_logger.setLevel(logging.INFO)
         root_logger.addHandler(handler)
 
-        
-
     def run(self, tasks, *, launcher:ElasticLauncher=None, run_id:str="none", out_dir:str=None, resume_from:str=None, seed=0, start_method="spawn", **kwds):
         """Run the tasks.
 
@@ -890,6 +890,7 @@ class HyperGraph:
         }
 
         if self.launcher.rank == 0:
+            import torch
             tqdm.write(f"Saving checkpoint to \"{save_to}\".")
             torch.save(_checkpoint, save_to)
 
@@ -904,7 +905,7 @@ class HyperGraph:
         Raises:
             ValueError: If the resume_from is not valid.
         """
-
+        import torch
         self.run_info.resume_from = resume_from
         if resume_from is None: return
         _checkpoint = torch.load(resume_from, map_location=self.launcher.assigned_device)
@@ -964,3 +965,18 @@ class HyperGraph:
                     task(*args)
             else:
                 get_logger().warning(f"A custom task `{task}` is not callable, skipping.")
+    
+    @overload
+    def __call__(self, *, train=True, steps=1, tags=["*"]) -> GraphOutputCache: ...
+    
+    @overload
+    def __call__(self, tasks) -> GraphOutputCache: ...
+
+    def __call__(self, tasks=None, train=True, steps=1, tags=["*"]) -> GraphOutputCache:
+        assert in_main_process(), "Eager Mode execution is only available in main process"
+        enable_auto_freeze(True)
+        if tasks is None:
+            tasks = [Task(train=train, steps=steps, tags=tags)]
+        self._run_impl(tasks, get_current_launcher(), None)
+        enable_auto_freeze(False)
+        return tasks[-1].egraph.cache
\ No newline at end of file
